@@ -16,7 +16,7 @@ class KateViewTest : public QObject
 
 public:
     KateViewTest();
-    ~KateViewTest();
+    ~KateViewTest() override;
 
 private Q_SLOTS:
     void testReloadMultipleViews();
@@ -28,11 +28,17 @@ private Q_SLOTS:
     void testDeselectByArrowKeys_data();
     void testDeselectByArrowKeys();
     void testKillline();
+    void testKeyDeleteBlockSelection();
     void testScrollPastEndOfDocument();
     void testFoldFirstLine();
     void testDragAndDrop();
     void testGotoMatchingBracket();
     void testFindSelected();
+    void testTransposeWord();
+    void testIndentBlockSelection();
+
+    void testFindMatchingFoldingMarker();
+    void testUpdateFoldingMarkersHighlighting();
 };
 
 #endif // KATE_VIEW_TEST_H
