@@ -11,7 +11,6 @@
 #include <katebuffer.h>
 #include <kateconfig.h>
 #include <katedocument.h>
-#include <kateglobal.h>
 #include <kateundomanager.h>
 #include <kateview.h>
 #include <kateviewinternal.h>
@@ -21,6 +20,7 @@
 #include <QApplication>
 #include <QKeyEvent>
 #include <QMouseEvent>
+#include <QStandardPaths>
 #include <QTest>
 
 using namespace KTextEditor;
@@ -44,7 +44,7 @@ DocAndView createDocAndView(const QString &text, int line, int column)
     doc->setText(text);
     auto view = new ViewPrivate(doc, nullptr);
     view->setCursorPosition({line, column});
-    return {doc, view};
+    return {.doc = doc, .view = view};
 }
 
 template<typename Cont>
@@ -56,7 +56,7 @@ bool isSorted(const Cont &c)
 MulticursorTest::MulticursorTest()
     : QObject()
 {
-    KTextEditor::EditorPrivate::enableUnitTestMode();
+    QStandardPaths::setTestModeEnabled(true);
 }
 
 MulticursorTest::~MulticursorTest()
@@ -163,8 +163,8 @@ void MulticursorTest::backspace()
     {
         // Mixed cursors, one doesn't have selection
         // the two below have selection
-        cursors.append({Cursor(1, 3), Range(1, 0, 1, 3)});
-        cursors.append({Cursor(2, 3), Range(2, 0, 2, 3)});
+        cursors.append({.pos = Cursor(1, 3), .range = Range(1, 0, 1, 3)});
+        cursors.append({.pos = Cursor(2, 3), .range = Range(2, 0, 2, 3)});
         view->addSecondaryCursorsWithSelection(cursors);
 
         // Pressing backspace should only remove selected text
@@ -199,8 +199,8 @@ void MulticursorTest::keyDelete()
     {
         // Mixed cursors, one doesn't have selection
         // the two below have selection
-        cursors.append({Cursor(1, 0), Range(1, 0, 1, 3)});
-        cursors.append({Cursor(2, 0), Range(2, 0, 2, 3)});
+        cursors.append({.pos = Cursor(1, 0), .range = Range(1, 0, 1, 3)});
+        cursors.append({.pos = Cursor(2, 0), .range = Range(2, 0, 2, 3)});
         view->addSecondaryCursorsWithSelection(cursors);
 
         // Pressing del should only remove selected text
@@ -333,6 +333,35 @@ void MulticursorTest::insertAutoBrackets()
     QCOMPARE(doc->text(), QStringLiteral("(hello\n(hello"));
 }
 
+void MulticursorTest::testInsertionWithCursorsAtSamePosition()
+{
+    auto [doc, view] = createDocAndView(QStringLiteral("one two thr fou"), 0, 0);
+    view->addSecondaryCursorsWithSelection({
+        // two cursors, same position different selections
+        KTextEditor::ViewPrivate::PlainSecondaryCursor{.pos = {0, 3}, .range = {0, 0, 0, 3}},
+        KTextEditor::ViewPrivate::PlainSecondaryCursor{.pos = {0, 3}, .range = {0, 3, 0, 6}},
+        // this one is at the same position as primary cursor
+        KTextEditor::ViewPrivate::PlainSecondaryCursor{.pos = {0, 11}, .range = {0, 8, 0, 11}},
+    });
+
+    view->setCursorPosition({0, 11});
+    view->setSelection({0, 11, 0, 13});
+    // uniquify
+    view->ensureUniqueCursors();
+    view->show();
+
+    // ensure nothing got removed, because all cursors have unique selections
+    QCOMPARE(view->secondaryCursors().size(), 3);
+    QCOMPARE(view->cursorPosition(), Cursor(0, 11));
+    QCOMPARE(view->selectionRange(), Range(0, 11, 0, 13));
+    doc->typeChars(view, QStringLiteral("a"));
+
+    QVERIFY(view->selectionRanges().empty());
+    const auto expectedCursors = QVector<KTextEditor::Cursor>{{0, 6}, {0, 1}, {0, 2}, {0, 5}};
+    QCOMPARE(view->cursorPosition(), Cursor(0, 6));
+    QCOMPARE(view->cursorPositions(), expectedCursors);
+}
+
 void MulticursorTest::testCreateMultiCursor()
 {
     auto [doc, view] = createDocAndView(QStringLiteral("foo\nbar\nfoo\n"), 0, 0);
@@ -965,6 +994,20 @@ void MulticursorTest::testSetGetSelections()
     }
 }
 
+void MulticursorTest::testIndent()
+{
+    auto [doc, view] = createDocAndView(QStringLiteral("foo\nbar\nfoo\nfoo"), 0, 0);
+    view->addSecondaryCursor({1, 0});
+    // set one cursor with selection
+    KTextEditor::ViewPrivate::PlainSecondaryCursor c;
+    c.pos = {2, 0};
+    c.range = {{2, 0}, {2, 2}};
+    view->addSecondaryCursorsWithSelection({c});
+    view->indent();
+    auto [expectedDoc, expectedView] = createDocAndView(QStringLiteral("    foo\n    bar\n    foo\nfoo"), 0, 0);
+    QCOMPARE(doc->text(), expectedDoc->text());
+}
+
 #include "moc_multicursortest.cpp"
 
 // kate: indent-mode cstyle; indent-width 4; replace-tabs on;
