@@ -11,9 +11,9 @@
 
 namespace Kate
 {
-TextBlock::TextBlock(TextBuffer *buffer, int startLine)
+TextBlock::TextBlock(TextBuffer *buffer, int index)
     : m_buffer(buffer)
-    , m_startLine(startLine)
+    , m_blockIndex(index)
 {
     // reserve the block size
     m_lines.reserve(BufferBlockSize);
@@ -29,33 +29,28 @@ TextBlock::~TextBlock()
     // it only is a hint for ranges for this block, not the storage of them
 }
 
-void TextBlock::setStartLine(int startLine)
+int TextBlock::startLine() const
 {
-    // allow only valid lines
-    Q_ASSERT(startLine >= 0);
-    Q_ASSERT(startLine < m_buffer->lines());
-
-    m_startLine = startLine;
+    return m_buffer->m_startLines[m_blockIndex];
 }
 
 TextLine TextBlock::line(int line) const
 {
     // right input
-    Q_ASSERT(line >= startLine());
-
+    Q_ASSERT(size_t(line) < m_lines.size());
     // get text line, at will bail out on out-of-range
-    return m_lines.at(line - startLine());
+    return m_lines.at(line);
 }
 
 void TextBlock::setLineMetaData(int line, const TextLine &textLine)
 {
     // right input
-    Q_ASSERT(line >= startLine());
+    Q_ASSERT(size_t(line) < m_lines.size());
 
     // set stuff, at will bail out on out-of-range
-    const QString originalText = m_lines.at(line - startLine()).text();
-    m_lines.at(line - startLine()) = textLine;
-    m_lines.at(line - startLine()).text() = originalText;
+    const QString originalText = m_lines.at(line).text();
+    m_lines.at(line) = textLine;
+    m_lines.at(line).text() = originalText;
 }
 
 void TextBlock::appendLine(const QString &textOfLine)
@@ -73,20 +68,16 @@ void TextBlock::clearLines()
 void TextBlock::text(QString &text) const
 {
     // combine all lines
-    for (size_t i = 0; i < m_lines.size(); ++i) {
-        // not first line, insert \n
-        if (i > 0 || startLine() > 0) {
-            text.append(QLatin1Char('\n'));
-        }
-
-        text.append(m_lines.at(i).text());
+    for (const auto &line : m_lines) {
+        text.append(line.text());
+        text.append(QLatin1Char('\n'));
     }
 }
 
 void TextBlock::wrapLine(const KTextEditor::Cursor position, int fixStartLinesStartIndex)
 {
     // calc internal line
-    int line = position.line() - startLine();
+    const int line = position.line() - startLine();
 
     // get text, copy, we might invalidate the reference
     const QString text = m_lines.at(line).text();
@@ -94,6 +85,7 @@ void TextBlock::wrapLine(const KTextEditor::Cursor position, int fixStartLinesSt
     // check if valid column
     Q_ASSERT(position.column() >= 0);
     Q_ASSERT(position.column() <= text.size());
+    Q_ASSERT(fixStartLinesStartIndex == m_blockIndex);
 
     // create new line and insert it
     m_lines.insert(m_lines.begin() + line + 1, TextLine());
@@ -123,7 +115,7 @@ void TextBlock::wrapLine(const KTextEditor::Cursor position, int fixStartLinesSt
     // fix all start lines
     // we need to do this NOW, else the range update will FAIL!
     // bug 313759
-    m_buffer->fixStartLines(fixStartLinesStartIndex);
+    m_buffer->fixStartLines(fixStartLinesStartIndex + 1, 1);
 
     // notify the text history
     m_buffer->history().wrapLine(position);
@@ -181,20 +173,13 @@ void TextBlock::wrapLine(const KTextEditor::Cursor position, int fixStartLinesSt
     // we might need to invalidate ranges or notify about their changes
     // checkValidity might trigger delete of the range!
     for (TextRange *range : std::as_const(changedRanges)) {
-        // we need to do updateRange to ALWAYS ensure the line => range and back cache is updated
-        // see MovingRangeTest::testLineWrapOrUnwrapUpdateRangeForLineCache
-        updateRange(range);
-
-        // in addition: ensure that we really invalidate bad ranges!
-        range->checkValidity(range->toLineRange());
+        // ensure that we really invalidate bad ranges!
+        range->checkValidity();
     }
 }
 
 void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLinesStartIndex)
 {
-    // calc internal line
-    line = line - startLine();
-
     // two possiblities: either first line of this block or later line
     if (line == 0) {
         // we need previous block with at least one line
@@ -203,7 +188,7 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
 
         // move last line of previous block to this one, might result in empty block
         const TextLine oldFirst = m_lines.at(0);
-        int lastLineOfPreviousBlock = previousBlock->lines() - 1;
+        const int lastLineOfPreviousBlock = previousBlock->lines() - 1;
         m_lines[0] = previousBlock->m_lines.back();
         previousBlock->m_lines.erase(previousBlock->m_lines.begin() + (previousBlock->lines() - 1));
 
@@ -216,13 +201,11 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
             m_lines[0].markAsModified(true);
         }
 
-        // patch startLine of this block
-        --m_startLine;
-
         // fix all start lines
         // we need to do this NOW, else the range update will FAIL!
         // bug 313759
-        m_buffer->fixStartLines(fixStartLinesStartIndex);
+        Q_ASSERT(fixStartLinesStartIndex + 1 == m_blockIndex);
+        m_buffer->fixStartLines(fixStartLinesStartIndex + 1, -1);
 
         // notify the text history in advance
         m_buffer->history().unwrapLine(startLine() + line, oldSizeOfPreviousLine);
@@ -279,12 +262,8 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         // we might need to invalidate ranges or notify about their changes
         // checkValidity might trigger delete of the range!
         for (TextRange *range : std::as_const(changedRanges)) {
-            // update both blocks
-            updateRange(range);
-            previousBlock->updateRange(range);
-
             // afterwards check validity, might delete this range!
-            range->checkValidity(range->toLineRange());
+            range->checkValidity();
         }
 
         // be done
@@ -310,7 +289,7 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
     // fix all start lines
     // we need to do this NOW, else the range update will FAIL!
     // bug 313759
-    m_buffer->fixStartLines(fixStartLinesStartIndex);
+    m_buffer->fixStartLines(fixStartLinesStartIndex + 1, -1);
 
     // notify the text history in advance
     m_buffer->history().unwrapLine(startLine() + line, oldSizeOfPreviousLine);
@@ -351,12 +330,8 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
     // we might need to invalidate ranges or notify about their changes
     // checkValidity might trigger delete of the range!
     for (TextRange *range : std::as_const(changedRanges)) {
-        // we need to do updateRange to ALWAYS ensure the line => range and back cache is updated
-        // see MovingRangeTest::testLineWrapOrUnwrapUpdateRangeForLineCache
-        updateRange(range);
-
-        // in addition: ensure that we really invalidate bad ranges!
-        range->checkValidity(range->toLineRange());
+        // ensure that we really invalidate bad ranges!
+        range->checkValidity();
     }
 }
 
@@ -427,7 +402,7 @@ void TextBlock::insertText(const KTextEditor::Cursor position, const QString &te
     // we might need to invalidate ranges or notify about their changes
     // checkValidity might trigger delete of the range!
     for (TextRange *range : std::as_const(changedRanges)) {
-        range->checkValidity(range->toLineRange());
+        range->checkValidity();
     }
 }
 
@@ -498,7 +473,7 @@ void TextBlock::removeText(KTextEditor::Range range, QString &removedText)
     // we might need to invalidate ranges or notify about their changes
     // checkValidity might trigger delete of the range!
     for (TextRange *range : std::as_const(changedRanges)) {
-        range->checkValidity(range->toLineRange());
+        range->checkValidity();
     }
 }
 
@@ -514,13 +489,10 @@ void TextBlock::debugPrint(int blockIndex) const
     }
 }
 
-TextBlock *TextBlock::splitBlock(int fromLine)
+void TextBlock::splitBlock(int fromLine, TextBlock *newBlock)
 {
     // half the block
-    int linesOfNewBlock = lines() - fromLine;
-
-    // create and insert new block
-    TextBlock *newBlock = new TextBlock(m_buffer, startLine() + fromLine);
+    const int linesOfNewBlock = lines() - fromLine;
 
     // move lines
     newBlock->m_lines.reserve(linesOfNewBlock);
@@ -534,6 +506,7 @@ TextBlock *TextBlock::splitBlock(int fromLine)
     m_lines.resize(fromLine);
 
     // move cursors
+    QSet<Kate::TextRange *> ranges;
     for (auto it = m_cursors.begin(); it != m_cursors.end();) {
         auto cursor = *it;
         if (cursor->lineInBlock() >= fromLine) {
@@ -543,40 +516,42 @@ TextBlock *TextBlock::splitBlock(int fromLine)
             // add to new, remove from current
             newBlock->m_cursors.insert(cursor);
             it = m_cursors.erase(it);
+            if (cursor->kateRange()) {
+                ranges.insert(cursor->kateRange());
+            }
         } else {
             // keep in current
             ++it;
         }
     }
 
-    // fix ALL ranges!
-    // copy is necessary as update range may modify the uncached ranges
-    std::vector<TextRange *> allRanges;
-    allRanges.reserve(m_uncachedRanges.size() + m_cachedLineForRanges.size());
-    std::for_each(m_cachedLineForRanges.keyBegin(), m_cachedLineForRanges.keyEnd(), [&allRanges](TextRange *range) {
-        allRanges.push_back(range);
-    });
-    allRanges.insert(allRanges.end(), m_uncachedRanges.begin(), m_uncachedRanges.end());
-    for (TextRange *range : allRanges) {
-        // update both blocks
-        updateRange(range);
-        newBlock->updateRange(range);
+    for (auto range : std::as_const(ranges)) {
+        if (range->spansMultipleBlocks()) {
+            m_buffer->addMultilineRange(range);
+        }
     }
-
-    // return the new generated block
-    return newBlock;
 }
 
 void TextBlock::mergeBlock(TextBlock *targetBlock)
 {
     // move cursors, do this first, now still lines() count is correct for target
+    QSet<Kate::TextRange *> ranges;
     for (TextCursor *cursor : m_cursors) {
         cursor->m_line = cursor->lineInBlock() + targetBlock->lines();
         cursor->m_block = targetBlock;
         targetBlock->m_cursors.insert(cursor);
+        if (cursor->kateRange()) {
+            ranges.insert(cursor->kateRange());
+        }
     }
     m_cursors.clear();
 
+    for (auto range : std::as_const(ranges)) {
+        if (!range->spansMultipleBlocks()) {
+            m_buffer->removeMultilineRange(range);
+        }
+    }
+
     // move lines
     targetBlock->m_lines.reserve(targetBlock->lines() + lines());
     for (size_t i = 0; i < m_lines.size(); ++i) {
@@ -584,20 +559,6 @@ void TextBlock::mergeBlock(TextBlock *targetBlock)
     }
     targetBlock->m_blockSize += m_blockSize;
     clearLines();
-
-    // fix ALL ranges!
-    // copy is necessary as update range may modify the uncached ranges
-    std::vector<TextRange *> allRanges;
-    allRanges.reserve(m_uncachedRanges.size() + m_cachedLineForRanges.size());
-    std::for_each(m_cachedLineForRanges.keyBegin(), m_cachedLineForRanges.keyEnd(), [&allRanges](TextRange *range) {
-        allRanges.push_back(range);
-    });
-    allRanges.insert(allRanges.end(), m_uncachedRanges.begin(), m_uncachedRanges.end());
-    for (TextRange *range : allRanges) {
-        // update both blocks
-        updateRange(range);
-        targetBlock->updateRange(range);
-    }
 }
 
 void TextBlock::deleteBlockContent()
@@ -650,44 +611,46 @@ void TextBlock::clearBlockContent(TextBlock *targetBlock)
 
 QList<TextRange *> TextBlock::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
 {
-    const auto cachedRanges = cachedRangesForLine(line);
     QList<TextRange *> ranges;
-    ranges.reserve(m_uncachedRanges.size() + (cachedRanges ? cachedRanges->size() : 0));
     rangesForLine(line, view, rangesWithAttributeOnly, ranges);
     return ranges;
 }
 
-void TextBlock::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
+void TextBlock::rangesForLine(const int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
 {
-    const auto cachedRanges = cachedRangesForLine(line);
     outRanges.clear();
-
-    auto predicate = [line, view, rangesWithAttributeOnly](TextRange *range) {
+    const int lineInBlock = line - startLine(); // line number in block
+    for (auto cursor : std::as_const(m_cursors)) {
+        if (!cursor->kateRange()) {
+            continue;
+        }
+        auto range = cursor->kateRange();
         if (rangesWithAttributeOnly && !range->hasAttribute()) {
-            return false;
+            continue;
         }
 
         // we want ranges for no view, but this one's attribute is only valid for views
         if (!view && range->attributeOnlyForViews()) {
-            return false;
+            continue;
         }
 
         // the range's attribute is not valid for this view
         if (range->view() && range->view() != view) {
-            return false;
+            continue;
         }
 
+        // simple case
+        if (cursor->lineInBlock() == lineInBlock) {
+            outRanges.append(range);
+        }
         // if line is in the range, ok
-        if (range->startInternal().lineInternal() <= line && line <= range->endInternal().lineInternal()) {
-            return true;
+        else if (range->startInternal().lineInternal() <= line && line <= range->endInternal().lineInternal()) {
+            outRanges.append(range);
         }
-        return false;
-    };
-
-    if (cachedRanges) {
-        std::copy_if(cachedRanges->begin(), cachedRanges->end(), std::back_inserter(outRanges), predicate);
     }
-    std::copy_if(m_uncachedRanges.begin(), m_uncachedRanges.end(), std::back_inserter(outRanges), predicate);
+    std::sort(outRanges.begin(), outRanges.end());
+    auto it = std::unique(outRanges.begin(), outRanges.end());
+    outRanges.erase(it, outRanges.end());
 }
 
 void TextBlock::markModifiedLinesAsSaved()
@@ -699,88 +662,4 @@ void TextBlock::markModifiedLinesAsSaved()
         }
     }
 }
-
-void TextBlock::updateRange(TextRange *range)
-{
-    // get some simple facts about our nice range
-    const int startLine = range->startInternal().lineInternal();
-    const int endLine = range->endInternal().lineInternal();
-    const bool isSingleLine = startLine == endLine;
-
-    // perhaps remove range and be done
-    if ((endLine < m_startLine) || (startLine >= (m_startLine + lines()))) {
-        removeRange(range);
-        return;
-    }
-
-    // The range is still a single-line range, and is still cached to the correct line.
-    if (isSingleLine) {
-        auto it = m_cachedLineForRanges.find(range);
-        if (it != m_cachedLineForRanges.end() && it.value() == startLine - m_startLine) {
-            return;
-        }
-    }
-
-    // The range is still a multi-line range, and is already in the correct set.
-    if (!isSingleLine && m_uncachedRanges.contains(range)) {
-        return;
-    }
-
-    // remove, if already there!
-    removeRange(range);
-
-    // simple case: multi-line range
-    if (!isSingleLine) {
-        // The range cannot be cached per line, as it spans multiple lines
-        m_uncachedRanges.append(range);
-        return;
-    }
-
-    // The range is contained by a single line, put it into the line-cache
-    const int lineOffset = startLine - m_startLine;
-
-    // enlarge cache if needed
-    if (m_cachedRangesForLine.size() <= (size_t)lineOffset) {
-        m_cachedRangesForLine.resize(lineOffset + 1);
-    }
-
-    // insert into mapping
-    if (!m_cachedRangesForLine[lineOffset].contains(range)) {
-        m_cachedRangesForLine[lineOffset].push_back(range);
-    }
-    m_cachedLineForRanges[range] = lineOffset;
-}
-
-void TextBlock::removeRange(TextRange *range)
-{
-    // uncached range? remove it and be done
-    int pos = m_uncachedRanges.indexOf(range);
-    if (pos != -1) {
-        m_uncachedRanges.remove(pos);
-        // must be only uncached!
-        Q_ASSERT(m_cachedLineForRanges.find(range) == m_cachedLineForRanges.end());
-        return;
-    }
-
-    // cached range?
-    auto it = m_cachedLineForRanges.find(range);
-    if (it != m_cachedLineForRanges.end()) {
-        // must be only cached!
-        Q_ASSERT(!m_uncachedRanges.contains(range));
-
-        int line = it.value();
-
-        // query the range from cache, must be there
-        int idx = m_cachedRangesForLine[line].indexOf(range);
-        Q_ASSERT(idx != -1);
-
-        // remove it and be done
-        m_cachedRangesForLine[line].remove(idx);
-        m_cachedLineForRanges.erase(it);
-        return;
-    }
-
-    // else: range was not for this block, just do nothing, removeRange should be "safe" to use
-}
-
 }
