@@ -125,9 +125,11 @@ void TextBuffer::clear()
     // kill all buffer blocks
     qDeleteAll(m_blocks);
     m_blocks.clear();
+    m_startLines.clear();
 
     // insert one block with one empty line
     m_blocks.push_back(newBlock);
+    m_startLines.push_back(0);
 
     // reset lines and last used block
     m_lines = 1;
@@ -154,7 +156,7 @@ TextLine TextBuffer::line(int line) const
     int blockIndex = blockForLine(line);
 
     // get line
-    return m_blocks.at(blockIndex)->line(line);
+    return m_blocks.at(blockIndex)->line(line - m_startLines[blockIndex]);
 }
 
 void TextBuffer::setLineMetaData(int line, const TextLine &textLine)
@@ -163,30 +165,30 @@ void TextBuffer::setLineMetaData(int line, const TextLine &textLine)
     int blockIndex = blockForLine(line);
 
     // get line
-    return m_blocks.at(blockIndex)->setLineMetaData(line, textLine);
+    return m_blocks.at(blockIndex)->setLineMetaData(line - m_startLines[blockIndex], textLine);
 }
 
 int TextBuffer::cursorToOffset(KTextEditor::Cursor c) const
 {
-    if (!c.isValid() || c > document()->documentEnd()) {
+    if ((c.line() < 0) || (c.line() >= lines())) {
         return -1;
     }
 
     int off = 0;
-    for (auto block : m_blocks) {
-        if (block->startLine() + block->lines() < c.line()) {
-            off += block->blockSize();
-        } else {
-            int start = block->startLine();
-            int end = start + block->lines();
-            for (int line = start; line < end; ++line) {
-                if (line >= c.line()) {
-                    off += qMin(c.column(), block->lineLength(line));
-                    return off;
-                }
-                off += block->lineLength(line) + 1;
-            }
+    const int blockIndex = blockForLine(c.line());
+    for (auto it = m_blocks.begin(), end = m_blocks.begin() + blockIndex; it != end; ++it) {
+        off += (*it)->blockSize();
+    }
+
+    auto block = m_blocks[blockIndex];
+    int start = block->startLine();
+    int end = start + block->lines();
+    for (int line = start; line < end; ++line) {
+        if (line >= c.line()) {
+            off += qMin(c.column(), block->lineLength(line));
+            return off;
         }
+        off += block->lineLength(line) + 1;
     }
 
     Q_ASSERT(false);
@@ -224,13 +226,14 @@ QString TextBuffer::text() const
     for (auto b : m_blocks) {
         size += b->blockSize();
     }
-    size -= 1; // remove -1, last newline
     text.reserve(size);
+    size -= 1; // remove -1, last newline
 
     // combine all blocks
-    for (TextBlock *block : std::as_const(m_blocks)) {
+    for (TextBlock *block : m_blocks) {
         block->text(text);
     }
+    text.chop(1); // remove last \n
 
     Q_ASSERT(size == text.size());
     return text;
@@ -339,13 +342,16 @@ void TextBuffer::unwrapLine(int line)
     int blockIndex = blockForLine(line);
 
     // is this the first line in the block?
-    bool firstLineInBlock = (line == m_blocks.at(blockIndex)->startLine());
+    const int blockStartLine = m_startLines[blockIndex];
+    const bool firstLineInBlock = line == blockStartLine;
 
     // let the block handle the unwrapLine
     // this can either lead to one line less in this block or the previous one
     // the previous one could even end up with zero lines
     // this call will trigger fixStartLines
-    m_blocks.at(blockIndex)->unwrapLine(line, (blockIndex > 0) ? m_blocks.at(blockIndex - 1) : nullptr, firstLineInBlock ? (blockIndex - 1) : blockIndex);
+
+    m_blocks.at(blockIndex)
+        ->unwrapLine(line - blockStartLine, (blockIndex > 0) ? m_blocks.at(blockIndex - 1) : nullptr, firstLineInBlock ? (blockIndex - 1) : blockIndex);
     --m_lines;
 
     // decrement index for later fixup, if we modified the block in front of the found one
@@ -464,24 +470,21 @@ int TextBuffer::blockForLine(int line) const
         b = m_blocks.size() - 1;
     }
 
-    auto block = m_blocks[b];
-    if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+    if (m_startLines[b] <= line && line < m_startLines[b] + m_blocks[b]->lines()) {
         return b;
     }
 
-    if (block->startLine() > line) {
+    if (m_startLines[b] > line) {
         for (int i = b - 1; i >= 0; --i) {
-            auto block = m_blocks[i];
-            if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+            if (m_startLines[i] <= line && line < m_startLines[i] + m_blocks[i]->lines()) {
                 return i;
             }
         }
     }
 
-    if (block->startLine() < line || (block->lines() == 0)) {
+    if (m_startLines[b] < line || (m_blocks[b]->lines() == 0)) {
         for (size_t i = b + 1; i < m_blocks.size(); ++i) {
-            auto block = m_blocks[i];
-            if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+            if (m_startLines[i] <= line && line < m_startLines[i] + m_blocks[i]->lines()) {
                 return i;
             }
         }
@@ -491,24 +494,15 @@ int TextBuffer::blockForLine(int line) const
     return -1;
 }
 
-void TextBuffer::fixStartLines(int startBlock)
+void TextBuffer::fixStartLines(int startBlock, int value)
 {
     // only allow valid start block
     Q_ASSERT(startBlock >= 0);
-    Q_ASSERT(startBlock < (int)m_blocks.size());
-
-    // new start line for next block
-    TextBlock *block = m_blocks.at(startBlock);
-    int newStartLine = block->startLine() + block->lines();
-
+    Q_ASSERT(startBlock <= (int)m_startLines.size());
     // fixup block
-    for (size_t index = startBlock + 1; index < m_blocks.size(); ++index) {
-        // set new start line
-        block = m_blocks.at(index);
-        block->setStartLine(newStartLine);
-
-        // calculate next start line
-        newStartLine += block->lines();
+    for (auto it = m_startLines.begin() + startBlock, end = m_startLines.end(); it != end; ++it) {
+        // move start line by given value
+        *it += value;
     }
 }
 
@@ -522,10 +516,18 @@ void TextBuffer::balanceBlock(int index)
         // half the block
         int halfSize = blockToBalance->lines() / 2;
 
-        // create and insert new block behind current one, already set right start line
-        TextBlock *newBlock = blockToBalance->splitBlock(halfSize);
-        Q_ASSERT(newBlock);
+        // create and insert new block after current one, already set right start line
+        const int newBlockStartLine = m_startLines[index] + halfSize;
+        TextBlock *newBlock = new TextBlock(this, index + 1);
         m_blocks.insert(m_blocks.begin() + index + 1, newBlock);
+        m_startLines.insert(m_startLines.begin() + index + 1, newBlockStartLine);
+
+        // adjust block indexes
+        for (auto it = m_blocks.begin() + index, end = m_blocks.end(); it != end; ++it) {
+            (*it)->setBlockIndex(index++);
+        }
+
+        blockToBalance->splitBlock(halfSize, newBlock);
 
         // split is done
         return;
@@ -536,6 +538,15 @@ void TextBuffer::balanceBlock(int index)
     // if only one block, no chance to unite
     // same if this is first block, we always append to previous one
     if (index == 0) {
+        // remove the block if its empty
+        if (blockToBalance->lines() == 0) {
+            m_blocks.erase(m_blocks.begin());
+            m_startLines.erase(m_startLines.begin());
+            Q_ASSERT(m_startLines[0] == 0);
+            for (auto it = m_blocks.begin(), end = m_blocks.end(); it != end; ++it) {
+                (*it)->setBlockIndex(index++);
+            }
+        }
         return;
     }
 
@@ -553,6 +564,14 @@ void TextBuffer::balanceBlock(int index)
     // delete old block
     delete blockToBalance;
     m_blocks.erase(m_blocks.begin() + index);
+    m_startLines.erase(m_startLines.begin() + index);
+
+    for (auto it = m_blocks.begin() + index, end = m_blocks.end(); it != end; ++it) {
+        (*it)->setBlockIndex(index++);
+    }
+
+    Q_ASSERT(index == (int)m_blocks.size());
+    Q_ASSERT(m_blocks.size() == m_startLines.size());
 }
 
 void TextBuffer::debugPrint(const QString &title) const
@@ -593,9 +612,11 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
             delete block;
         }
         m_blocks.resize(1);
+        m_startLines.resize(1);
 
         // remove lines in first block
         m_blocks.back()->clearLines();
+        m_startLines.back() = 0;
         m_lines = 0;
 
         // reset error flags
@@ -637,7 +658,10 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
 
             // ensure blocks aren't too large
             if (m_blocks.back()->lines() >= BufferBlockSize) {
-                m_blocks.push_back(new TextBlock(this, m_blocks.back()->startLine() + m_blocks.back()->lines()));
+                int index = (int)m_blocks.size();
+                int startLine = m_blocks.back()->startLine() + m_blocks.back()->lines();
+                m_blocks.push_back(new TextBlock(this, index));
+                m_startLines.push_back(startLine);
             }
 
             // append line to last block
@@ -938,6 +962,53 @@ void TextBuffer::markModifiedLinesAsSaved()
         block->markModifiedLinesAsSaved();
     }
 }
+
+void TextBuffer::addMultilineRange(TextRange *range)
+{
+    auto it = std::find(m_multilineRanges.begin(), m_multilineRanges.end(), range);
+    if (it == m_multilineRanges.end()) {
+        m_multilineRanges.push_back(range);
+        return;
+    }
+}
+
+void TextBuffer::removeMultilineRange(TextRange *range)
+{
+    m_multilineRanges.erase(std::remove(m_multilineRanges.begin(), m_multilineRanges.end(), range), m_multilineRanges.end());
+}
+
+bool TextBuffer::hasMultlineRange(KTextEditor::MovingRange *range) const
+{
+    return std::find(m_multilineRanges.begin(), m_multilineRanges.end(), range) != m_multilineRanges.end();
+}
+
+void TextBuffer::rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
+{
+    // get block, this will assert on invalid line
+    const int blockIndex = blockForLine(line);
+    m_blocks.at(blockIndex)->rangesForLine(line, view, rangesWithAttributeOnly, outRanges);
+    // printf("Requested range for line %d, available %d\n", (int)line, (int)m_multilineRanges.size());
+    for (auto range : m_multilineRanges) {
+        if (rangesWithAttributeOnly && !range->hasAttribute()) {
+            continue;
+        }
+
+        // we want ranges for no view, but this one's attribute is only valid for views
+        if (!view && range->attributeOnlyForViews()) {
+            continue;
+        }
+
+        // the range's attribute is not valid for this view
+        if (range->view() && range->view() != view) {
+            continue;
+        }
+
+        // if line is in the range, ok
+        else if (range->startInternal().lineInternal() <= line && line <= range->endInternal().lineInternal()) {
+            outRanges.append(range);
+        }
+    }
+}
 }
 
 #include "moc_katetextbuffer.cpp"
