@@ -418,7 +418,7 @@ private:
      * @param startBlock index of block from which we start to fix
      */
     KTEXTEDITOR_NO_EXPORT
-    void fixStartLines(int startBlock);
+    void fixStartLines(int startBlock, int value);
 
     /**
      * Balance the given block. Look if it is too small or too large.
@@ -491,16 +491,12 @@ public:
     QList<TextRange *> rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
     {
         // get block, this will assert on invalid line
-        const int blockIndex = blockForLine(line);
-        return m_blocks.at(blockIndex)->rangesForLine(line, view, rangesWithAttributeOnly);
+        QList<TextRange *> outRanges;
+        rangesForLine(line, view, rangesWithAttributeOnly, outRanges);
+        return outRanges;
     }
 
-    void rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
-    {
-        // get block, this will assert on invalid line
-        const int blockIndex = blockForLine(line);
-        return m_blocks.at(blockIndex)->rangesForLine(line, view, rangesWithAttributeOnly, outRanges);
-    }
+    void rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const;
 
     /**
      * Check if the given range pointer is still valid.
@@ -516,6 +512,13 @@ public:
      */
     void invalidateRanges();
 
+    /**
+     * Add/Remove a multiline range that spans multiple blocks
+     */
+    KTEXTEDITOR_NO_EXPORT void addMultilineRange(TextRange *range);
+    KTEXTEDITOR_NO_EXPORT void removeMultilineRange(TextRange *range);
+    bool hasMultlineRange(KTextEditor::MovingRange *range) const;
+
     //
     // checksum handling
     //
@@ -548,6 +551,11 @@ private:
      */
     TextHistory m_history;
 
+    /**
+     * List of starting lines of the blocks in m_blocks
+     */
+    std::vector<int> m_startLines;
+
     /**
      * List of blocks which contain the lines of this buffer
      */
@@ -599,6 +607,11 @@ private:
      */
     QSet<TextRange *> m_ranges;
 
+    /**
+     * Multiline ranges that span multiple blocks
+     */
+    std::vector<TextRange *> m_multilineRanges;
+
     /**
      * Encoding prober type to use
      */
