@@ -21,6 +21,8 @@
 #include "katetextline.h"
 #include <ktexteditor_export.h>
 
+#include <span>
+
 class KJob;
 class KateTemplateHandler;
 namespace KTextEditor
@@ -233,7 +235,7 @@ public:
      * @param s string to be inserted
      * @return true on success
      */
-    bool editInsertText(int line, int col, const QString &s);
+    bool editInsertText(int line, int col, const QString &s, bool notify = true);
 
     /**
      * Remove a string in the given line/column
@@ -264,7 +266,7 @@ public:
      * @param newLineAdded return value is true, if new line was added (may be 0)
      * @return true on success
      */
-    bool editWrapLine(int line, int col, bool newLine = true, bool *newLineAdded = nullptr);
+    bool editWrapLine(int line, int col, bool newLine = true, bool *newLineAdded = nullptr, bool notify = true);
 
     /**
      * Unwrap @p line. If @p removeLine is true, we force to join the lines. If
@@ -281,7 +283,7 @@ public:
      * @param s string to insert
      * @return true on success
      */
-    bool editInsertLine(int line, const QString &s);
+    bool editInsertLine(int line, const QString &s, bool notify = true);
 
     /**
      * Remove a line
@@ -378,9 +380,6 @@ public:
 protected:
     KateUndoManager *const m_undoManager;
 
-Q_SIGNALS:
-    void undoChanged();
-
 public:
     QList<KTextEditor::Range> searchText(KTextEditor::Range range, const QString &pattern, const KTextEditor::SearchOptions options) const;
 
@@ -1172,29 +1171,37 @@ private:
 private:
     /**
      * read dir config file
+     *
+     * if @p view is set, then variables will
+     * only be applied to the given view and nothing else.
      */
     KTEXTEDITOR_NO_EXPORT
-    void readDirConfig();
+    void readDirConfig(KTextEditor::ViewPrivate *v = nullptr);
 
     /**
       Reads all the variables in the document.
       Called when opening/saving a document
       Returns true if variables were read
+      if @p view is set, then variables will
+      only be applied to the given view and nothing else.
     */
-    bool readVariables(bool onlyViewAndRenderer = false);
+    bool readVariables(KTextEditor::ViewPrivate *view = nullptr);
     // exported for katedocument_test
 
     /**
       Reads and applies the variables in a single line
       TODO registered variables gets saved in a [map]
+
+      if @p view is set, then variables will
+      only be applied to the given view and nothing else.
     */
-    void readVariableLine(const QString &t, bool onlyViewAndRenderer = false);
+    void readVariableLine(const QString &t, KTextEditor::ViewPrivate *view = nullptr);
     // exported for katedocument_test
     /**
       Sets a view variable in all the views.
     */
     KTEXTEDITOR_NO_EXPORT
-    void setViewVariable(const QString &var, const QString &val);
+    void setViewVariable(const QString &var, const QString &val, std::span<KTextEditor::View *> views);
     /**
       @return weather a string value could be converted
       to a bool value as supported.
