@@ -793,7 +793,7 @@ public:
      * \return the number of characters in the document
      * \see lines()
      */
-    virtual int totalCharacters() const = 0;
+    virtual qsizetype totalCharacters() const = 0;
 
     /**
      * Returns if the document is empty.
@@ -1424,7 +1424,7 @@ public:
         OnDiskUnmodified = 0, ///< Not modified
         OnDiskModified = 1, ///< The file was modified on disk
         OnDiskCreated = 2, ///< The file was created on disk
-        OnDiskDeleted = 3 ///< The file was deleted on disk
+        OnDiskDeleted = 3 ///< The file was deleted or moved on disk
     };
 
     /**
