@@ -18,6 +18,7 @@
 
 #include <KLocalizedString>
 
+#include <QCollator>
 #include <QDateTime>
 #include <QRegularExpression>
 
@@ -601,3 +602,111 @@ bool KateCommands::Date::exec(KTextEditor::View *view, const QString &cmd, QStri
 }
 
 // END Date
+
+KateCommands::EditingCommands *KateCommands::EditingCommands::s_instance = nullptr;
+
+KateCommands::EditingCommands::EditingCommands()
+    : KTextEditor::Command({
+        QStringLiteral("uniq"),
+        QStringLiteral("sortuniq"),
+        QStringLiteral("natsort"),
+        QStringLiteral("sort"),
+    })
+{
+}
+
+QList<KateCommands::EditingCommands::EditingCommand> KateCommands::EditingCommands::allCommands()
+{
+    static QList<EditingCommand> cmds{
+        {.name = i18n("Remove Duplicate Lines"), .cmd = QStringLiteral("uniq")},
+        {.name = i18n("Remove Duplicates and Sort Text Alphabetically"), .cmd = QStringLiteral("sortuniq")},
+        {.name = i18n("Sort Text Naturally"), .cmd = QStringLiteral("natsort")},
+        {.name = i18n("Sort Selected Text Alphabetically"), .cmd = QStringLiteral("sort")},
+    };
+    return cmds;
+}
+
+bool KateCommands::EditingCommands::exec(KTextEditor::View *view, const QString &cmd, QString &, const KTextEditor::Range &)
+{
+    auto getDocAndRange = [view]() -> std::pair<KTextEditor::Document *, KTextEditor::Range> {
+        KTextEditor::Range r = view->document()->documentRange();
+        if (view->selection()) {
+            return {view->document(), view->selectionRange()};
+        }
+        return {view->document(), r};
+    };
+
+    auto apply = [](KTextEditor::Range range, KTextEditor::Document *doc, const QStringList &lines) {
+        if (range == doc->documentRange()) {
+            doc->setText(lines);
+        } else {
+            doc->replaceText(range, lines);
+        }
+    };
+
+    if (cmd == QStringLiteral("uniq")) {
+        const auto [doc, range] = getDocAndRange();
+        const QStringList lines = doc->textLines(range);
+        QSet<QString> seenLines;
+        QStringList uniqueLines;
+        for (const auto &line : lines) {
+            auto it = seenLines.find(line);
+            if (it == seenLines.end()) {
+                seenLines.insert(line);
+                uniqueLines.push_back(line);
+            }
+        }
+        apply(range, doc, uniqueLines);
+        return true;
+    } else if (cmd == QStringLiteral("sortuniq")) {
+        const auto [doc, range] = getDocAndRange();
+        QStringList lines = doc->textLines(range);
+        std::sort(lines.begin(), lines.end());
+        auto it = std::unique(lines.begin(), lines.end());
+        if (it != lines.end()) {
+            lines.erase(it, lines.end());
+        }
+        apply(range, doc, lines);
+        return true;
+    } else if (cmd == QStringLiteral("natsort")) {
+        const auto [doc, range] = getDocAndRange();
+        QStringList lines = doc->textLines(range);
+        QCollator col;
+        col.setNumericMode(true);
+        std::sort(lines.begin(), lines.end(), col);
+        apply(range, doc, lines);
+        return true;
+    } else if (cmd == QStringLiteral("sort")) {
+        const auto [doc, range] = getDocAndRange();
+        QStringList lines = doc->textLines(range);
+        std::sort(lines.begin(), lines.end());
+        apply(range, doc, lines);
+        return true;
+    }
+
+    return false;
+}
+bool KateCommands::EditingCommands::help(class KTextEditor::View *, const QString &cmd, QString &msg)
+{
+    if (cmd == QStringLiteral("sort")) {
+        msg = i18n(
+            "<p>sort</p>"
+            "<p>Sort the selected text or whole document if there is no selection</p>");
+    } else if (cmd == QStringLiteral("uniq")) {
+        msg = i18n(
+            "<p>uniq</p>"
+            "<p>Remove duplicate lines from the selected text or whole document if there is no selection.</p>");
+    } else if (cmd == QStringLiteral("sortuniq")) {
+        msg = i18n(
+            "<p>sortuniq</p>"
+            "<p>Sort the selected text or whole document and then remove all duplicate lines.</p>");
+    } else if (cmd == QStringLiteral("natsort")) {
+        msg = i18n(
+            "<p>natsort</p>"
+            "<p>Sort the selected text or whole document in natural order.<br>Here is an example to show the difference to the normal sort "
+            "method:<br>sort(a10, a1, a2) => a1, a10, a2<br>natsort(a10, a1, a2) => a1, a2, a10</p>");
+    } else {
+        return false;
+    }
+    return true;
+}
